@@ -24,12 +24,12 @@
 
 using namespace Rcpp;
 
-std::vector<std::string> decodeURIComponent(std::vector<std::string> value);
+Rcpp::CharacterVector decodeURIComponent(Rcpp::CharacterVector value);
 RcppExport SEXP httpuv_decodeURIComponent(SEXP valueSEXP) {
 BEGIN_RCPP
     Rcpp::RObject rcpp_result_gen;
     Rcpp::RNGScope rcpp_rngScope_gen;
-    Rcpp::traits::input_parameter< std::vector<std::string> >::type value(valueSEXP);
+    Rcpp::CharacterVector value(valueSEXP);
     rcpp_result_gen = Rcpp::wrap(decodeURIComponent(value));
     return rcpp_result_gen;
 END_RCPP
